%% POP
lambda = 1.064e-6;
mm = 1e-3;

% Coordinates and optics
OpticalLayout;                    % load the layout from the model

idPRM = strmatch('PRM', OpticNames);
R_prm = layout(idPRM).ROC;         % optic (and beam) radius of curvature)
w_prm = layout(idPRM).waist;       % laser beam radius
tPRM =  layout(idPRM).thickness;   % optic thickness
nPRM = 1.45;                       % index of refraction

idPR2 = strmatch('PR2', OpticNames);   
R_PR2 = layout(idPR2).ROC;            % Ditto for the PR2
tPR2  = layout(idPR2).thickness;
nPR2  = 1.45;

R_M1  = 1.5;                      % from M. Arain
%%
% distances
dPRM_PR2 = norm(layout(idPR2).position - layout(idPRM).position);
dPR2_M1  = 2.5;                   % from M. Arain 


% build beam path to PR2 (remember, POP comes from PRM, not BS)
prm = beamPath;
%prm.seedq = beamq.beamSizeAndR(w_prm, -R_prm);                           % input waist
prm.seedq = beamq.beamWidthAndR(w_prm, -R_prm);                           % input waist
prm.seedz = 0;
prm.addComponent(component.curvedMirror(R_PR2, dPRM_PR2, 'PR2'));
tmp = prm.qPropagate(prm.component('PR2').z-1e-6);     % The q immediately before PR2

q_in = real(tmp.q) + 1i*imag(tmp.q);      % Reverse the beam propagation

pop = beamPath;
pop.seedq = beamq(q_in);
pop.seedz = 0;
%pop.addComponent(component.dielectric(-R_PR2, inf, tPR2, nPR2, 1e-6, 'PR2'));
pop.addComponent(component.lens(-R_PR2/(nPR2-1), 1e-6, 'PR2'));

f1 =  0.343;    % CVI lens
f2 = -0.057;    % CVI lens
f1 = 333e-3;    % r=150mm fused silica lens
f2 = -55.6e-3;  % r=-25mm fused silica lens
f3 = 0.05;



pop.addComponent(component.lens(f1, dPR2_M1-tPR2, 'L1'));
% Lens 2 has 0.3mm position error
pop.addComponent(component.lens(f2, dPR2_M1-tPR2+275.3e-3, 'L2'));

%%
pop_optimized = pop.duplicate();
pop_optimized.targetWaist(250e-6, dPR2_M1-tPR2+0.3);

pop_optimized = pop_optimized.optimizePath('L2',[2.7 4], ...
                       'target', [2.9 5]);

outRayleigh = pop_optimized.qPropagate(pop_optimized.targetz).rayleighRange;


% Add QPDs to the non-optimized path as the same position as the optimized path
pop.addComponent(component.curvedMirror(inf, pop_optimized.targetz-outRayleigh, 'QPD1'));
pop.addComponent(component.curvedMirror(inf, pop_optimized.targetz+outRayleigh, 'QPD2'));

% Add PRM
pop.addComponent(component.curvedMirror(-11.0, -16.604, 'PRM'));
%%
disp(' ')
disp('========== Optimized Path Component List ==============')
display(pop.components)

fprintf(1,'PRM to QPD1 = %.0f\n', pop.gouySeperation('PRM', 'QPD1'))
fprintf(1,'PR2 to QPD1 = %.0f\n', pop.gouySeperation('PR2', 'QPD1'))
fprintf(1,'PRM to QPD2 = %.0f\n', pop.gouySeperation('PRM', 'QPD2'))
fprintf(1,'PR2 to QPD2 = %.0f\n', pop.gouySeperation('PR2', 'QPD2'))

%%
% plot beam
figure(1)
clf
zz = -2:0.01:4;
pop.plotSummary(zz);
shg
saveas(gcf, 'POP_DC_beampath.pdf','pdf');

