%% 
clear all
close all
%% Initialization
% Measured upstream (source) parameterxs, measured from the launcher lens mount.
wu=[2.487, 2.619]*1e-3;
zu=[7.020, 10.847]; % waist is 7 or 10 m upstream of the last launcher lens. Positive means that the waist is upstream.
dimensions={'X', 'Y'};

% Nominal lens placement
% lens1
z1=(299+108+340)*1e-3;
f1=333.6e-3;

% lens2
nominaldistance = 275e-3;
z2 = z1 + nominaldistance;
f2=-55.6e-3;

% QPD positions
zQ1= z2+374e-3;
zQ2= zQ1+300e-3;

% Measured downstream parameters. This is used as a target.
wd=[.628 .639]*1e-3/2;
zd_MM=[-0.717 -0.750]; %waist position from the modemaster head, positive means downstream of the modemaster

MM2L2 = 329.5e-3; % Measured distance from the downstream face of the L2 holder to the MM face
zd=z2 + MM2L2; % Position of MM

%% Don't modify things from here

for ii=1:length(dimensions)
    % First we propagate the upstream beam to the downstream using nominal lens
    % distance, just for plotting.

    nominalPath=beamPath;
    nominalPath.seedq = beamq.beamWaistAndZ(wu(ii), zu(ii));
    nominalPath.seedz = -1e-6;

    nominalPath.addComponent(component.lens(f1, z1, 'L1'));
    nominalPath.addComponent(component.lens(f2, z2, 'L2'));

    nominalPath.addComponent(component.flatMirror(zQ1, 'QPD1'));
    nominalPath.addComponent(component.flatMirror(zQ2, 'QPD2'));
    nominalPath.addComponent(component.flatMirror(zd, 'MM'));
    z=(z1-1):0.01:(z2+1);

    % Set the target.
    nominalPath.targetz = zd; % this is the position of the MM head
    nominalPath.targetq = beamq.beamWaistAndZ(wd(ii), -zd_MM);
    % The sign of zd_MM should be flipped because, if zd is positive (i.e. the
    % waist is downstream of the MM), from ALM's point of view, at MM position, 
    % the beam is not yet reached the waist.

    % Now match the upstream and downstream. Ideally we want to do this by moving L2 and target mode together, but ALM cannot do it.
    % We'll just move L1. This should be good enough.

    measuredPathWithFit = nominalPath.optimizePath('L1', [z1-0.02, z1+0.02]);
    figure
    measuredPathWithFit.plotSummary(z)
    title([dimensions{ii} ' L1 position optimized for the measured data'])
    subplot 211
    text(0, -1e-3, [dimensions{ii} ' overlap: ' num2str(measuredPathWithFit.targetOverlap)])
  
    lensspacing=measuredPathWithFit.component('L2').z-measuredPathWithFit.component('L1').z;
    if nominaldistance > lensspacing
        text(0, -2e-3, [dimensions{ii} ' spacing: ' num2str(1000*nominaldistance) '-' num2str(1000*(nominaldistance-lensspacing)) 'mm'])
    else
        text(0, -2e-3, [dimensions{ii} ' spacing: ' num2str(1000*nominaldistance) '+' num2str(1000*(-nominaldistance+lensspacing)) 'mm'])
    end
end